-- Sample usage of Narrow_Xml to parse a XML file
with Text_Io;
with Simple_Xml.Xml_Io;
with GWindows.Application;
with GWindows.Windows;
with GWindows.Buttons;
with Ada.Characters.Handling;

procedure Show_Form is
  use Simple_Xml.Xml_Io;
  use GWindows.Windows;
  use GWindows.Buttons;
  use Text_Io;
  use Ada.Characters.Handling;

  My_Window    : GWindows.Windows.Window_Type;
  My_Buttons   : array (1 .. 10) of GWindows.Buttons.Button_Type;
  Button_Count : Integer := 0;

  Level : Integer := 0;    -- Current node level
  procedure Get_Position
   (Attributes  : in Xml_Attributes;
    Left, Top, Width, Height : out Integer) is
  begin
    Left   := Integer'value (Attribute(Attributes, "Left"));
    Top    := Integer'value (Attribute(Attributes, "Top"));
    Width  := Integer'value (Attribute(Attributes, "Width"));
    Height := Integer'value (Attribute(Attributes, "Height"));
  end Get_Position;

  procedure Add_Button
    (Attributes : in Xml_Attributes) is
    Left, Top, Width, Height : Integer;
  begin
    Get_Position (Attributes, Left, Top, Width, Height);
    Button_Count := Button_Count +1;
    Create  (My_Buttons (Button_Count), My_Window, "Button", Left, Top, Width, Height);
    Visible (My_Buttons (Button_Count), True);
  end Add_Button;

  procedure Add_Form
    (Attributes : in Xml_Attributes) is
    Left, Top, Width, Height : Integer;
  begin
    Get_Position (Attributes, Left, Top, Width, Height);
    Create  (My_Window, "Window", Left, Top, Width, Height);
  end Add_Form;

  procedure Set_Title
    (Title : in String) is
  begin
    case Level is
    when 2      => Text (My_Window, Title);
    when 3      => Text (My_Buttons (Button_Count), Title);
    when others => null;
    end case;
  end Set_Title;

  procedure Open_Node (Tag        : in String;
                       Attributes : in Xml_Attributes;
                       Value      : in String;
                       Depth      : in Integer) is
  begin
    Level := Depth;
    if    Tag = "FORM"
    then
      Add_Form   (Attributes);
    elsif Tag = "BUTTON"
    then
      Add_Button (Attributes);
    elsif Tag = "TITLE"
    then
      Set_Title  (Value);
    end if;
  end Open_Node;

  procedure Close_Node (Depth : in Integer)  is
  begin
    Level := Depth;
  end Close_Node;

  procedure Note_Version (Version  : in String) is
  begin
    null;
  end Note_Version;

  procedure Note_Encoding (Encoding  : in String) is
  begin
    null;
  end Note_Encoding;

  procedure On_Error is
  begin
    null;
  end On_Error;

  procedure Parse_File is new Parse (Open_Node, Close_Node, Note_Version, Note_Encoding, On_Error);

begin

  Text_Io.Put ("Parsing Sample_Form.xml ... ");
  Parse_File (File_Name => "Sample_Form.xml");
  Text_Io.Put_Line ("[done]");
  Show (My_Window);
  GWindows.Application.Message_Loop;

end Show_Form;

